@@ -7,24 +7,22 @@ Created on Sat Oct 24 15:57:27 2020
 """
 
 from ._database import unitsNetwork
-from ._dictionaries import dictionary
+from ._dictionaries import dictionary, temperatureRatioConversions
 from ._searches import BFS as _BFS, printPath as _printPath
 from ._errors import NoConversionFound
 import numpy as np
 from functools import reduce
 
-__version__ = '0.2.1'
-__release__ = 20220822
+__version__ = '0.2.7'
+__release__ = 20220908
 
 #defaultPrintConversionPath = unitsNetwork.print  # True
 
 
 def convertible(fromUnit, toUnit, PrintPath=False):
     try:
-        if converter(1, fromUnit, toUnit, PrintPath) is None:
-            return False
-        else:
-            return True
+        converter(1, fromUnit, toUnit, False)
+        return True
     except:
         return False
 
@@ -51,6 +49,27 @@ def convertUnit_old(value, fromUnit, toUnit, PrintConversionPath=False):
 
 
 def convertUnit(value, fromUnit, toUnit, PrintConversionPath=False):
+    """
+    returns the received value (integer, float, array, Series, DataFrame, etc)
+    transformed from the units 'fromUnit' to the units 'toUnits.
+
+    Parameters
+    ----------
+    value : int, float, array, Series, DataFrame, etc
+        the value to be converted.
+    fromUnit : str
+        the units of the provided value.
+    toUnit : str
+        the units to convert the value.
+    PrintConversionPath : bool, optional
+        Set to True to show the path used for conversion. The default is False.
+
+    Returns
+    -------
+    converted_value : int, float, array, Series, DataFrame ...
+        the converted value
+
+    """
     if type(PrintConversionPath) is not bool:
         if type(PrintConversionPath) in [int, float]:
             if PrintConversionPath > 1:
@@ -167,9 +186,28 @@ def _get_conversion(value, fromUnit, toUnit, PrintConversionPath=None, getPath=F
     if fromUnit in dictionary['date'] and toUnit in dictionary['date']:
         return (lambda x: x) if value is None else value
 
-    ## dimensionless units does not require conversion
-    if fromUnit.lower() in dictionary['dimensionless'] and toUnit.lower() in dictionary['dimensionless']:
+    ## special case for temperature ratios
+    if '/' in fromUnit and len(fromUnit.split('/'))==2 and fromUnit.split('/')[0] in dictionary['temperature'] \
+        and '/' in toUnit and len(toUnit.split('/'))==2 and toUnit.split('/')[0] in dictionary['temperature']:
+            t1, d1 = fromUnit.split('/')
+            t2, d2 = toUnit.split('/')
+            num = temperatureRatioConversions[(t1, t2)]
+            den = _get_conversion(1, d1, d2, PrintConversionPath=PrintConversionPath, getPath=getPath)
+            if num is None or den is None:
+                return None  # raise NoConversionFound("from '" + str(d1) + "' to '" + str(d2) + "'")
+            return (lambda x: x * num / den) if value is None else (value * num / den)
+
+    ## from dimensionless/None to some units or viceversa (to allow assign units to dimensionless numbers)
+    if fromUnit is None or toUnit is None:
         return (lambda x: x) if value is None else value
+    if (fromUnit.lower() in dictionary['dimensionless'] or toUnit.lower() in dictionary['dimensionless']) and (fromUnit is None or toUnit is None):
+        return (lambda x: x) if value is None else value
+
+    ## from dimensionless to percentage or viceversa
+    if (fromUnit is None or fromUnit.lower() in dictionary['dimensionless']) and toUnit.lower() in dictionary['percentage']:
+        return (lambda x: x * 100) if value is None else value * 100
+    if fromUnit.lower() in dictionary['percentage'] and (toUnit is None or toUnit.lower() in dictionary['dimensionless']):
+        return (lambda x: x / 100) if value is None else value / 100
 
     ## from dimensionless to ratio of same units
     if fromUnit.lower() in dictionary['dimensionless'] and '/' in toUnit and len(toUnit.split('/')) == 2 and toUnit.lower().split('/')[0].strip(' ()') == toUnit.lower().split('/')[1].strip(' ()'):
@@ -183,12 +221,6 @@ def _get_conversion(value, fromUnit, toUnit, PrintConversionPath=None, getPath=F
     if ('/' in fromUnit and len(fromUnit.split('/')) == 2) and ('/' in toUnit and len(toUnit.split('/')) == 2) and (fromUnit.split('/')[0].strip() == toUnit.split('/')[1].strip()) and (fromUnit.split('/')[1].strip() == toUnit.split('/')[0].strip()):
         return (lambda x: 1/x) if value is None else 1/value
 
-    ## from dimensionless/None to some units or viceversa (to allow assign units to dimensionless numbers)
-    if fromUnit is None or toUnit is None:
-        return (lambda x: x) if value is None else value
-    if fromUnit.lower() in dictionary['dimensionless'] or toUnit.lower() in dictionary['dimensionless']:
-        return (lambda x: x) if value is None else value
-
     # check if already solved and memorized
     if not getPath and (fromUnit, toUnit) in unitsNetwork.Memory:
         conversionLambda = unitsNetwork.Memory[(fromUnit,toUnit)]
